% Function to count the maximal and minimal number of transcripts to cover ASG.

% First, tie up each source to a supersource and each sink to a supersink
ASG = S;

ASG = [zeros(exons,1) ASG];
ASG_supersource_row = zeros(1,exons+1);
ASG_supersource_row(I+1) = pstart;
ASG = [ASG_supersource_row; ASG];

ASG = [ASG; zeros(1,exons+1)];
ASG_supersink_column = zeros(exons+2,1);
ASG_supersink_column(T+1) = 1;
ASG = [ASG ASG_supersink_column];

% Maximum
maximum_transcripts = nnz(ASG) - exons

% Minimum

% Construct an edge covering
edge_count = 0*ASG; % A count of the number of paths passing through each edge
                    % edge_count(i,j) = n, where (i,j) corresponds to the
                    % appropriate edge in ASG.
[x,y] = find(ASG); % Co-ordinates in ASG of all edges
[xx, yy] = find(edge_count == 0);
uncovered_edges = intersect([x y], [xx yy], 'rows');

total_paths = 0;

while ~isempty(uncovered_edges)
    
    total_paths = total_paths + 1;
    
    % First, find an uncovered edge
    newpath = [uncovered_edges(1,1) uncovered_edges(1,2)];  % We use matrix indices, not exon number
    edge_count(uncovered_edges(1,1),uncovered_edges(1,2)) = edge_count(uncovered_edges(1,1),uncovered_edges(1,2)) + 1;
    
    % Extend to a sink
    sink_found = 0;
    while sink_found == 0
        
        end_exon = newpath(end,2);
        next_edges = find(ASG(end_exon,:));
        
        if isempty(next_edges)
            sink_found = 1;
        else
            uncovered_next_edges = intersect(next_edges, find(edge_count(end_exon,:) == 0));
            if isempty(uncovered_next_edges)
                new_edge = next_edges(1);
            else
                new_edge = uncovered_next_edges(1);
            end
            
            newpath = [newpath; end_exon new_edge];
            edge_count(end_exon,new_edge) = edge_count(end_exon,new_edge) + 1;
        end
    end
    
    % Extend to a source
    source_found = 0;
    while source_found == 0
        % Find all upstream edges ending at the top of "newpath"
        if newpath(1,1) == 1
            source_found = 1;
        else
            prev_exons = find(ASG(:,newpath(1,1)));
            if isempty(prev_exons)
                disp('Error: the following edge cannot be linked any farther upstream');
                newpath(1,:)-1
                break;
            else
                edge_count(prev_exons(1),newpath(1,1)) = edge_count(prev_exons(1),newpath(1,1)) + 1;
                newpath = [prev_exons(1) newpath(1,1); newpath];
            end
        end
        
    end
    
    [xx, yy] = find(edge_count == 0);
    uncovered_edges = intersect([x y], [xx yy], 'rows');
end

% Now trim the edge counts, based on redundant transcripts.
% These are signified by a path from source to sink using only edges with
% edge_count > 1 or any edge in the wrong direction. (*)
% Use a breadth-first search (see "Introduction to Algorithms", Cormen et
% al. 1990)

search_failed = 0;
while search_failed == 0
    
    colour = zeros(size(ASG,1),1);  % 0 = white, 1 = grey, 2 = black.
    pi = 0*colour;                  % Upstream marker
    Q = [1];    % Queue. Start at supersource (entries correspond to rows of ASG).

    while ~isempty(Q)
        u = Q(1);
        % Find adjacent vertices to u that we can get to by rules (*).
        adj1 = find(edge_count(u,:) > 1);
        adj2 = find(edge_count(:,u));
        adj = union(adj1,adj2);

        if ~isempty(adj)
            for i = 1:length(adj)
                if colour(adj(i)) == 0
                    colour(adj(i)) = 1;
                    pi(adj(i)) = u;
                    Q = [Q; adj(i)];
                end
            end
        end
        Q(1) = [];
        colour(u) = 2;
    end

    % Check whether we reached supersink, and reconstruct path
    if colour(end) == 2
        path = [size(ASG,1)];
        while path(1) ~= 1
            path = [pi(path(1)); path];
        end
        total_paths = total_paths - 1;

        % Update edge counts. For those traversed forwards, deduct one.
        % For those traversed backwards add 1.
        for i = 1:length(path)-1
            if path(i+1) > path(i)
                edge_count(path(i),path(i+1)) = edge_count(path(i),path(i+1)) - 1;
            elseif path(i+1) < path(i)
                edge_count(path(i+1),path(i)) = edge_count(path(i+1),path(i)) + 1;
            else
                disp('Error')
                break
            end
        end
    else
        search_failed = 1;
    end
end

minimum_transcripts = total_paths