clear all;

filename = input('Enter filename for gene: ', 's');
[fid,message] = fopen(filename, 'r');
if (fid == -1)
    disp(message)
end

% Get exons
exons = textscan(fid, '%d', 1,'commentstyle', '#');
exons = exons{1,1} - 2; % Convert from cell array to number and deduct supersource and supersink

% Get I and pstart
nI = textscan(fid, '%d', 1,'commentstyle', '#');
nI = nI{1,1};
starts = textscan(fid, '%d %f', nI,'commentstyle', '#');
I = starts{1,1};
pstart = starts{1,2}/sum(starts{1,2});
cumpstart = [0; cumsum(pstart)];

% Get S, T, pin, pout
S = zeros(exons); T = []; pin = zeros(exons,1); pout = zeros(exons,1);
W = zeros(exons); % edge weights
eW = zeros(exons); % edge weights (external edges only)
inout_indicator = zeros(exons,1); % Indicator for whether we go in at start of exon.
num_in = zeros(exons,1);  % num_in(i) = number in at exon i
num_out = zeros(exons,1); % num_out(i) = number out between exons i and i+1
curr_num_in = 0; curr_num_out = 0;

for i=1:exons
   attached = textscan(fid, '%s', 1, 'commentstyle', '#');
   attached = attached{1,1};
   
   outdegree = textscan(fid, '%d', 1,'commentstyle', '#');
   outdegree = outdegree{1,1};
   
   nexts =  textscan(fid, '%d %f', outdegree,'commentstyle', '#');
   next_exons = nexts{1,1};
   weight_next_exons = nexts{1,2};
   p_next_exons = weight_next_exons/sum(weight_next_exons);

   if ~isempty(find(next_exons == exons+1))
       T = [T; i]; % Append to T those exons which can reach the supersink
   else
       S(i,next_exons) = p_next_exons;
   end
   
   if inout_indicator(i) == 1
       pin(i) = sum(eW(:,i))/curr_num_out;
       curr_num_out = curr_num_out - sum(eW(:,i));
       curr_num_in = curr_num_in + sum(eW(:,i));
   end

   num_in(i) = sum(weight_next_exons);
   if ~ismember(i,T)
       W(i,next_exons) = weight_next_exons;
       if strcmp(attached,'Y')
           next_exon_index = find(next_exons == i+1);
           eW(i,next_exons) = weight_next_exons;
           eW(i,next_exons(next_exon_index)) = 0; % We don't care about the weights of edges which are 'in'
           inout_indicator(setdiff(next_exons,i+1)) = 1;
           pout(i) = sum(p_next_exons) - p_next_exons(next_exon_index);
           curr_num_out = curr_num_out + sum(weight_next_exons) - weight_next_exons(next_exon_index);
           curr_num_in = weight_next_exons(next_exon_index);
           num_out(i) = curr_num_out;
       else
           eW(i,next_exons) = weight_next_exons;
           inout_indicator(next_exons) = 1;
           pout(i) = 1;
           curr_num_out = curr_num_out + sum(weight_next_exons);
           curr_num_in = 0;
           num_out(i) = curr_num_out;
       end
   else
       pout(i) = 0;
       curr_num_in = 0; 
       num_out(i) = curr_num_out;
   end
end

next_entry = textscan(fid, '%d', 1, 'commentstyle', '#');
next_entry = next_entry{1,1}; % Should be the 0 for the outdegree of supersink
if next_entry ~= 0
   disp('Error in reading file')
end

fclose(fid);

% Check entries are valid
if length(pin) ~= exons
    disp(['Error - pin does not match number of exons']);
end

if length(pout) ~= exons
    disp(['Error - pout does not match number of exons']);
end

if nnz(~ismember(I,1:exons)) > 0
    disp(['Error - element of I out of range']);
end

if nnz(~ismember(T,1:exons)) > 0
    disp(['Error - element of T out of range']);
end

if length(I) ~= length(pstart)
    disp(['Error - I and pstart differ in length']);
end

for i=1:exons
    if ~ismember(i,T) && (sum(S(i,:)) <= 0.9999 || sum(S(i,:)) > 1)   % We want to test equality to 1 but there are rounding errors
        disp(['Error - probabilities of S(', num2str(i),',:) do not sum to 1']);
    end
end