from sys import stdin, stdout
from random import uniform
from types import StringTypes
from digraph import Node, Edge, DiGraph
import digraph

class Node(Node):
  def __init__(self, index = None):
    digraph.Node.__init__(self, index)
    self._attached = False
  def copy(self):
    return Node(self.get_index())
  def set_attached(self):
    self._attached = True
  def clear_attached(self):
    self._attached = False
  def attached(self):
    return self._attached
  def __sample(self, edges):
    # Use binary search to locate x in a[i..j]
    def bsearch(i, j, x, a):
      if i + 1 >= j:
        return i
      else:
        k = (i + j) / 2
        if a[k - 1] > x:
          return bsearch(i, k, x, a)
        else:
          return bsearch(k, j, x, a)
    # Compute prefix sum of weights
    w = [e.get_weight() for e in edges]
    for i in xrange(1, len(edges)):
      w[i] = w[i - 1] + w[i]
    # Use a random number to select an edge
    return edges[bsearch(0, len(edges), uniform(0, w[-1]), w)]
  def sample_incoming(self):
    return self.__sample(self._incoming)
  def sample_outgoing(self):
    return self.__sample(self._outgoing)
  def normalise_outgoing(self):
    s = sum([e.get_weight() for e in self.outgoing()])
    if s != 0:
      for e in self.outgoing():
        e.set_weight(e.get_weight() / float(s))
  def normalise_incoming(self):
    s = sum([e.get_weight() for e in self.incoming()])
    if s != 0:
      for e in self.incoming():
        e.set_weight(e.get_weight() / float(s))

class WeightedEdge(Edge):
  def __init__(self, source = None, target = None, weight = 0):
    Edge.__init__(self, source, target)
    self._weight = weight
  def copy(self, source = None, target = None):
    return WeightedEdge(source, target, self.get_weight())
  def set_weight(self, weight):
    self._weight = weight
  def get_weight(self):
    return self._weight
  
class WeightedDiGraph(DiGraph):
  def copy(self):
    G = DiGraph.copy(self)
    H = WeightedDiGraph()
    H._V = G._V
    return H
  def read(self, infile = stdin):
    # Read next line from f, ignoring empty lines and removing comments;
    # return line as a list of tokens
    def nextline(f):
      # Convert s to a list of tokens, removing comments and returning
      # None if there are no non-white space symbols in s
      def cleanup_line(s):
        # Find next token in s
        def next_token(s):
          if s in ("", "\\"):
            # s is empty or consists solely of the escape character
            return "", None, False
          if s[0] == '\\':
            # Next symbol in s is escaped
            if s[1] == 'n':
              # Newline
              return s[2:], '\n', True
            if s[1] == 't':
              # Tab
              return s[2:], '\t', True
            # Other character; do no conversion
            return s[2:], s[1], True
          # Non-escaped symbol
          return s[1:], s[0], False
          ### End of next_token
        #
        t = []
        empty = True
        while s != "":
          s, u, v = next_token(s)
          if not v and u == "#":
            # Non-escaped # - start of comment, skip remainder of line
            break
          t.append((u, v))
          empty = empty and not v and u.isspace()
        if empty:
          return None
        else:
          return t
        ### End of cleanup_line
      #
      # Start by reading next line from f
      line = f.readline()
      if line != "":
        line = cleanup_line(line.strip())
        if line == None:
          # Line was empty or all comment - return next line instead
          return nextline(f)
        return line
      else:
        # End of file
        return None
      ### End of nextline
    # Convert a list of tokens to a list of white space separated elements
    def untoken(l):
      e = []
      i = 0
      j = 0
      while i < len(l):
        # Skip stretches of non-escaped white spaces
        while j < len(l) and not l[j][1] and l[j][0].isspace():
          j += 1
        i = j
        # Determine next block of non-white spaces
        while i < len(l) and (l[i][1] or not l[i][0].isspace()):
          i += 1
        if i > j:
          # Insert this block in list of elements
          e.append("".join([a[0] for a in l[j:i]]))
        j = i
      return e
    # Interpret next element of s as an integer; if s is empty, read new line
    # from infile
    def next_int(s, infile):
      if s == []:
        s = untoken(nextline(infile))
      return int(s[0]), s[1:]
    # Interpret next element of s a float; if s is empty, read new line
    # from infile
    def next_float(s, infile):
      if s == []:
        s = untoken(nextline(infile))
      return float(s[0]), s[1:]
    # Interpret next element of s as a string; if s is empty read new line
    # from infile
    def next_string(s, infile):
      if s == []:
        s = untoken(nextline(infile))
      return s[0], s[1:]
    # If infile is a name, assume it is a file name and open this file
    if type(infile) in StringTypes:
      infile = file(infile)
      close_at_exit = True
    else:
      close_at_exit = False
    # First line should be number of nodes
    n, s = next_int([], infile)
    self._V = []
    for i in xrange(n):
      self._V.append(Node(len(self._V)))
    # Rest of file should contain descriptions of each node
    for i in xrange(n):
      # First line of description should be outdegree of node
      m, s = next_string(s, infile)
      try:
        m = int(m)
      except:
        # Or it could be attachment to next node
        if m in "yY":
          self._V[i].set_attached()
        else:
          self._V[i].clear_attached()
        m, s = next_int(s, infile)
      # Read each of the target nodes in turn
      for j in xrange(m):
        t, s = next_int(s, infile)
        w, s = next_float(s, infile)
        WeightedEdge(self._V[i], self._V[t], w)
    # Check whether infile should be closed
    if close_at_exit:
      infile.close()
  # Output graph to outfile
  def write(self, outfile = stdout):
    if type(outfile) in StringTypes:
      outfile = file(outfile, 'w')
      close_at_exit = True
    else:
      close_at_exit = False
    # Output number of nodes
    outfile.write("# Size of graph\n" + str(len(self._V)) + '\n')
    # Output nodes
    tmp = {True: 'Y', False: 'N'}
    for v in self._V:
      if v not in [self._V[0], self._V[-1]]:
        outfile.write("\n# Node " + str(v.get_index()) + "\n# Attached to next fragment (Y/N)\n" + tmp[v.attached()] + "\n#  Outdegree\n" + str(v.outdegree()) + "\n#  Outgoing edge destinations\n")
      else:
        outfile.write("\n# Node " + str(v.get_index()) + "\n#  Outdegree\n" + str(v.outdegree()) + "\n#  Outgoing edge destinations\n")
      for e in v.outgoing():
        w = e.get_weight()
        if type(w) not in StringTypes:
          w = str(w)
        outfile.write(str(e.get_target().get_index()) + ' ' + w + '\n')
    if close_at_exit:
      outfile.close()
  # Create new node, add it to graph, and return it
  def add_node(self):
    self._V.append(Node(len(self._V)))
    return self._V[-1]
  # Add a weighted edge from s to t
  def add_edge(self, s, t, w = 0):
    if isinstance(s, Node) and isinstance(t, Node) and s in self._V and t in self._V:
      e = WeightedEdge(s, t, w)
      return e
    else:
      return None
  # Sample a path from s to t (default is from first node to last node)
  def sample_path(self, s = None, t = None):
    if s == None:
      s = self._V[0]
    if t == None:
      t = self._V[-1]
    if self.find_path(s, t) == None:
      return None
    elif s == t:
      return []
    # edges = filter(lambda e: self.find_path(e.get_target(), t) != None, s.outgoing())
    e = s.sample_outgoing()#s._Node__sample(edges)
    return [e] + self.sample_path(e.get_target(), t)
  def normalise_outgoing(self):
    for n in self.V():
      n.normalise_outgoing()
  def normalise_incoming(self):
    for n in self.V():
      n.normalise_incoming()
